env JWT_SECRET;
#expiration timeout in seconds
env TOKEN_EXPIRATION_TS;
env AD_TOKEN_EXPIRATION_TS;
#env BDE_API_DOMAIN;
env MGMT_CONSOLE_ROOT;

#user  nobody;
worker_processes  1;

error_log  stderr  debug;
error_log  /var/log/nginx_error.log  error;

#pid        logs/nginx.pid;

events {
    worker_connections  1024;
}


http {
    include       mime.types;
    default_type  application/octet-stream;

    # linux path
     lua_package_path "/usr/local/openresty/lualib/?.lua;;";

    # macOS path
    #lua_package_path "/usr/local/Cellar/openresty/1.11.2.5/nginx/lualib/?.lua;;";

    log_format  main  '$remote_addr - $remote_user [$time_local] "$request" '
                      '$status $body_bytes_sent "$http_referer" '
                      '"$http_user_agent" "$http_x_forwarded_for"';

    access_log  /var/log/nginx_http.log;

    sendfile        on;
    #tcp_nopush     on;

    #keepalive_timeout  0;
    keepalive_timeout  65;

    gzip  on;
    resolver        8.8.8.8;

    upstream conduit-server-app-api {
        #ip_hash;
        {% for conduit_server_and_port in CONDUIT_SERVERS.split(',') -%}
          server {{ conduit_server_and_port }};
        {% endfor %}
    }
    upstream grafana-server-host {
        #ip_hash
        {% for gf_server_and_port in GRAFANA_SERVER.split(',') -%}
          server {{ gf_server_and_port }};
        {% endfor %}
    }
    upstream kibana-server-host {
        #ip_hash
        {% for kibana_server_and_port in KIBANA_SERVER.split(',') -%}
          server {{ kibana_server_and_port }};
        {% endfor %}
    }
    upstream spark-server-host {
        #ip_hash
        {% for spark_server_and_port in SPARK_SERVER.split(',') -%}
          server {{ spark_server_and_port }};
        {% endfor %}
    }
    upstream upgrade-server-app {
        #ip_hash;
        {% for upgrade_server_and_port in UPGRADE_SERVER.split(',') -%}
          server {{ upgrade_server_and_port }};
        {% endfor %}
    }

{% if ENABLE_SSL.lower() == 'true' %}
    server {
        listen  80;
        server_name  localhost;
        server_tokens   off;

        set_by_lua $mgmt_console_root 'return os.getenv("MGMT_CONSOLE_ROOT") or "/var/www/mgmt_console" ';


        location /.well-known {
            root   $mgmt_console_root;
            try_files $uri $uri/ =404;
        }

        location / {
            return 301 https://$host$request_uri;
        }
    }
{% endif %}


    server {

    {% if ENABLE_SSL.lower() == 'true' %}

        listen  443  default_server ssl ;

        ssl_certificate /usr/local/etc/openresty/certs/ssl.crt;
        ssl_certificate_key /usr/local/etc/openresty/certs/ssl.key;

    {% else %}

        listen        80;

    {% endif %}

        #server_name  <%- BDE_API_DOMAIN %>;
        server_name localhost;

        set_by_lua $mgmt_console_root 'return os.getenv("MGMT_CONSOLE_ROOT") or "/var/www/mgmt_console" ';

        location /api/activedirectory-enabled {
            proxy_pass http://conduit-server-app-api/api/activedirectory-enabled;
        }

        location /api {

            access_by_lua '
                local jwt = require("nginx-jwt")
                local cjson = require("cjson");
                local now = os.time();
                local luajwt = require("luajwt")
                local alg = "HS256"
                jwt.auth({
                    expiration_ts=function (val) return now < val end
                })
                local auth_header = ngx.var.http_Authorization
                local _, _, token = string.find(auth_header, "Bearer%s+(.+)")
                local jwt_secret = os.getenv("JWT_SECRET") or "BLUEPRINTJWT" ;
                local user_obj, _ = luajwt.decode(token, jwt_secret, alg)
                local user_json = cjson.encode(user_obj)
                -- ngx.log(ngx.DEBUG, "User json: " .. user_json)
                ngx.req.set_header("X-AUTH-USER", user_json)
                ngx.req.set_header("X-AUTH-TOKEN", token)
            ';

            location /api/upgrade-service/ {
                proxy_read_timeout 600s;
                proxy_pass http://upgrade-server-app/;
            }

            location /api/backup-service/ {
                proxy_read_timeout 600s;
                proxy_pass http://upgrade-server-app/;
            }

            location /api/management/ {
                proxy_read_timeout 600s;
                proxy_pass http://upgrade-server-app/management/;
            }

            location /api/ {
                proxy_connect_timeout        {{ API_READ_TIMEOUT | default('120s') }};
                proxy_send_timeout           {{ API_READ_TIMEOUT | default('120s') }};
                proxy_read_timeout           {{ API_READ_TIMEOUT | default('120s') }};
                send_timeout                 {{ API_READ_TIMEOUT | default('120s') }};
                proxy_set_header Host            $http_host;
                proxy_set_header X-Forwarded-For $remote_addr;
                proxy_set_header X-Forwarded-Proto https;
                proxy_pass http://conduit-server-app-api;
            }

        }

        location /login/oauth2/auth/user {
            proxy_pass http://conduit-server-app-api;
            proxy_set_header Host            $http_host;
            proxy_set_header X-Forwarded-For $remote_addr;
            proxy_set_header X-Forwarded-Proto https;
        }

        location /login/oauth2/code/azure/subscription/validation {

            access_by_lua '
                res = ngx.location.capture("/_callAzureCallback", {method = ngx.HTTP_GET, always_forward_body = true, copy_all_vars = true} );

                local response_body = res.body;
                ngx.header["Content-type"] = "text/plain";
                ngx.status = res.status;
                ngx.log(ngx.DEBUG, res.status);

                local json = require("cjson");
                local user_json = json.decode(response_body);
                local redirectPath = user_json.url

                if res.status == 200 then
                    return ngx.redirect(redirectPath);
                end

                return ngx.redirect("/subscriptions?errorMessage=An error occurred during authentication");
            ';
        }

        location /login/oauth2/code/azure/user {

            access_by_lua '
                res = ngx.location.capture("/_callAzureCallback", {method = ngx.HTTP_GET, always_forward_body = true, copy_all_vars = true} );

                local response_body = res.body;
                ngx.header["Content-type"] = "text/plain";
                ngx.status = res.status;

                ngx.log(ngx.DEBUG, res.status);

                if res.status == 200 then
                    local luajwt = require("luajwt");
                    local json = require("cjson");
                    local alg = "HS256";
                    local user_json = json.decode(response_body);
                    local ad_token_expiration_ts = os.getenv("AD_TOKEN_EXPIRATION_TS") or "7776000" ;
                    local jwt_secret = os.getenv("JWT_SECRET") or "BLUEPRINTJWT" ;

                    local exptime = os.time() + tonumber(ad_token_expiration_ts);
                    ngx.log(ngx.DEBUG, "token_expiration_ts: " .. tostring(ad_token_expiration_ts) .. " exptime: " .. tostring(exptime) );

                    user_json["expiration_ts"] = exptime;
                    local token, err = luajwt.encode(user_json, jwt_secret, alg);

                    ngx.header["Set-Cookie"] = "jwt=" .. token .. "; path=/;  Max-Age=" .. tostring(ad_token_expiration_ts) .. ";";

                    return ngx.redirect("/");
                end

                if res.status == 403 then
                    return ngx.redirect("/rolesError");
                end

                return ngx.redirect("/");
            ';
        }

        location /_callAzureCallback {
            proxy_pass http://conduit-server-app-api$request_uri;
            proxy_set_header Host            $http_host;
            proxy_set_header X-Forwarded-For $remote_addr;
            proxy_set_header X-Forwarded-Proto https;

            access_by_lua '
                local luajwt = require("luajwt")
                if (ngx.status ~= 200) then
                    ngx.exit(ngx.status);
                else
                    ngx.header["Content-type"] = "text/html";
                    ngx.say("<H1>Authorized.</H1>");
                    ngx.exit(ngx.status);
                end
            ';
        }

        location /logout {

            access_by_lua '
                local jwt = require("nginx-jwt")
                local cjson = require("cjson");
                local now = os.time();
                local luajwt = require("luajwt")
                local alg = "HS256"
                jwt.auth({
                    expiration_ts=function (val) return now < val end
                })
                local auth_header = ngx.var.http_Authorization
                local _, _, token = string.find(auth_header, "Bearer%s+(.+)")
                local jwt_secret = os.getenv("JWT_SECRET") or "BLUEPRINTJWT" ;
                local user_obj, _ = luajwt.decode(token, jwt_secret, alg)
                local user_json = cjson.encode(user_obj)
                -- ngx.log(ngx.DEBUG, "User json: " .. user_json)
                ngx.req.set_header("X-AUTH-USER", user_json)
                ngx.req.set_header("X-AUTH-TOKEN", token)
            ';

            proxy_pass http://conduit-server-app-api/logout;
            proxy_set_header Host            $http_host;
            proxy_set_header X-Forwarded-For $remote_addr;
            proxy_set_header X-Forwarded-Proto https;
        }

        location /logout/oauth2 {
            proxy_pass http://conduit-server-app-api;
            proxy_set_header Host            $http_host;
            proxy_set_header X-Forwarded-For $remote_addr;
            proxy_set_header X-Forwarded-Proto https;
        }

        location /odata {
            proxy_set_header Host            $host;
            proxy_set_header X-Forwarded-For $remote_addr;
            proxy_pass http://conduit-server-app-api;
        }

        location /register_self {
            proxy_pass http://conduit-server-app-api/register_self;
        }
        location /email-verification {
            proxy_pass http://conduit-server-app-api/email-verification;
        }

        location /forgot_password {
            proxy_pass http://conduit-server-app-api/forgot_password;
        }

        location /reset_password {
            proxy_pass http://conduit-server-app-api/reset_password;
        }

        location /auth {
            access_by_lua '
                local request_body = ngx.req.read_body();
                res = ngx.location.capture("/_callauthserver", { method = ngx.HTTP_POST, body = request_body });
                local response_body = res.body;
                ngx.header["Content-type"] = "text/plain";
                ngx.status = res.status;
                if res.status == 200 then
                    local luajwt = require("luajwt");
                    local json = require("cjson");
                    local alg = "HS256";
                    local user_json = json.decode(response_body);
                    local token_expiration_ts = os.getenv("TOKEN_EXPIRATION_TS") or "7776000" ;
                    local jwt_secret = os.getenv("JWT_SECRET") or "BLUEPRINTJWT" ;

                    local exptime = os.time() + tonumber(token_expiration_ts);
                    ngx.log(ngx.DEBUG, "token_expiration_ts: " .. tostring(token_expiration_ts) .. " exptime: " .. tostring(exptime) );

                    user_json["expiration_ts"] = exptime;
                    local token, err = luajwt.encode(user_json, jwt_secret, alg);

                    ngx.header["X-AUTH-TOKEN"] = token;
                    value = { jwtToken = token }
                    ngx.say(json.encode(value));
                    return res.status;
                else
                    ngx.say(res.body);
                    return res.status;
                end
            ';
        }

        location /provisioning {
            proxy_pass http://conduit-server-app-api/provisioning;
        }

        location /grafana/ {

            set $user "admin";

            add_header X-uri "$user";
            proxy_set_header X-WEBAUTH-USER $user;

            proxy_pass http://grafana-server-host/;
        }

        location /kibana/ {
            proxy_pass http://kibana-server-host/;
        }

        location /swagger/ {
            proxy_pass http://conduit-server-app-api/swagger/;
        }

        location /swagger-ui/ {
            proxy_pass http://conduit-server-app-api/swagger-ui/;
        }

        location /sparkui/ {
            proxy_pass http://spark-server-host/;
            proxy_set_header Accept-Encoding '';
            # XXX: spark UI doesn't add spark.ui.proxyBase prefix to some links on UI thus we have to rewrite url in these cases
            sub_filter '="/proxy/' '="/sparkui/proxy/';
            sub_filter_once off;
        }

        location /_callauthserver {

            proxy_pass http://conduit-server-app-api/authenticate;

            access_by_lua '
                local luajwt = require("luajwt")
                if (ngx.status ~= 200) then
                    ngx.exit(ngx.status);
                else
                    ngx.header["Content-type"] = "text/html";
                    ngx.say("<H1>Authorized.</H1>");
                    ngx.exit(ngx.status);
                end
            ';
            # allow only request only from localhost
            allow   127.0.0.1;
            deny    all;
        }

        #error_page  404              /404.html;

        # redirect server error pages to the static page /50x.html
        #
        error_page   500 502 503 504  /50x.html;
        location = /50x.html {
            root   html;
        }

        location / {
            root   $mgmt_console_root;
            try_files $uri $uri/ /index.html;
            expires       0;
            add_header    Cache-Control  public;
            add_header    Cache-Control  no-store;
            add_header    Cache-Control  no-cache;
        }
    }
}
