#!/bin/bash

set -e

while getopts f:u:p:l:i:v:c:s:t: option
do
case "${option}"
in
u) USER=${OPTARG};;
p) PASS=${OPTARG};;
f) FQDN=${OPTARG};; # http(s)://domain_name
l) LT=${OPTARG};;
i) LST=${OPTARG};;
v) VALID=${OPTARG};;
c) CONN=${OPTARG};;
s) SUBID=${OPTARG};;
t) TENANTID=${OPTARG};;
esac
done

echo "Creating license for ${FQDN}..."

AUTHTOKEN=""
if [[ ! -z "${PASS}" ]]; then
    #Log in to new isntance of BDE Server before we generate license
    echo "Checking credentials to ${FQDN} "
    OUTPUT=`curl --silent --show-error -k -X POST ${FQDN}/auth -H 'Content-Type: application/json' -d '{ "email": "'${USER}'", "password": "'${PASS}'" }'`

    if [[ ! $(jq -r  .jwtToken <<< ${OUTPUT}) == "null" ]]; then
        AUTHTOKEN=$(jq -r  .jwtToken <<< ${OUTPUT})
        echo "Successfully authenticated to ${FQDN} "
    else
        echo "Authentication failed:  $(jq -r  .message <<< ${OUTPUT})  "
        exit 4
    fi
fi

JSON=`curl -X POST https://conduit-services.westus2.cloudapp.azure.com/license-server/license -H 'Authorization: '${LST} -H 'Content-Type: application/json' -d '{ "clientUUID": "'${FQDN}'", "validForMonths": '${VALID}', "connectors": '${CONN}', "subscriptionId": "'${SUBID}'", "tenantId": "'${TENANTID}'", "licenseToken": "'${LT}'" }'`

ERROR=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict.get("error", ""))
EOF
)
MSG=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict.get("message", ""))
EOF
)

STATUS=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict.get("status", -1))
EOF
)

if [[ ! -z "${ERROR}" ]]; then

    >&2 echo  "ERROR: Failed to generate license for  user: ${USER}, deployment: ${FQDN} , error: ${ERROR},  message: ${MSG}"
    exit $STATUS
fi
LICENSE=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict["license"])
EOF
)

PUBLICKEY=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict["publicKey"])
EOF
)

#Post license values to instance
OUTCOME=`curl -k -X POST ${FQDN}/api/license -H 'Authorization: Bearer '${AUTHTOKEN} -H 'Content-Type: application/json' -H 'Accept: text/plain' -d '{ "license": "'${LICENSE}'", "publicKey": "'${PUBLICKEY}'" }'`

echo $OUTCOME
