#!/bin/bash

set -e

while getopts i:d:p: option
do
case "${option}"
in
p) PASS=${OPTARG};;
i) LST=${OPTARG};;
d) LICENSE_JSON_FILE=${OPTARG};;
esac
done

if [[ ! -f "${LICENSE_JSON_FILE}" ]]; then
    echo "License data file not found at ${LICENSE_JSON_FILE} "
    exit 2
fi

if [[ -z "${LST}" ]]; then
   echo "Invalid License server authentication token"
   exit 3
fi
#XXX: clientUUID holds deployment url at the moment
FQDN=$(jq -r .clientUUID ${LICENSE_JSON_FILE} )
USER=$(jq -r .adminEmail ${LICENSE_JSON_FILE} )
echo "Creating license for ${FQDN} "

AUTHTOKEN=""
DISPLAY_LICENSE=false
if [[ ! -z "${PASS}" ]]; then
    #Log in to new isntance of BDE Server before we generate license
    echo "Checking credentials to ${FQDN} "
    OUTPUT=`curl --silent --show-error -k -X POST ${FQDN}/auth -H 'Content-Type: application/json' -d '{ "email": "'${USER}'", "password": "'${PASS}'" }'`

    if [[ ! $(jq -r  .jwtToken <<< ${OUTPUT}) == "null" ]]; then
        AUTHTOKEN=$(jq -r  .jwtToken <<< ${OUTPUT})
        echo "Successfully authenticated to ${FQDN} "
    else
        DISPLAY_LICENSE=true
        echo "Authentication failed:  $(jq -r  .message <<< ${OUTPUT})  "
    fi
else
    DISPLAY_LICENSE=true
    echo "No authentication for ${FQDN} was provided, license info has to be registered manually "
fi

JSON=`curl -X POST https://conduit-services.westus2.cloudapp.azure.com/license-server/v2/license -H 'Authorization: '${LST} -H 'Content-Type: application/json' -d  @${LICENSE_JSON_FILE} `

ERROR=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict.get("error", ""))
EOF
)
MSG=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict.get("message", ""))
EOF
)

STATUS=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict.get("status", -1))
EOF
)

if [[ ! -z "${ERROR}" ]]; then

    >&2 echo  "ERROR: Failed to generate license for  user: ${USER}, deployment: ${FQDN} , error: ${ERROR},  message: ${MSG}"
    exit ${STATUS}
fi
LICENSE=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict["license"])
EOF
)

PUBLICKEY=$(python <<EOF
import sys
thisDict = ${JSON}
print(thisDict["publicKey"])
EOF
)
echo "License has been generated for ${FQDN}"
if [[ ${DISPLAY_LICENSE} == true ]]; then

    echo "=============== Your License key =================="
    echo ${LICENSE}
    echo "==================================================="
    echo "================ Your Public key =================="
    echo ${PUBLICKEY}
    echo "==================================================="

fi

#Log in to new isntance of BDE Server
if [[ ! -z "${AUTHTOKEN}" ]]; then
    #Post license values to instance
    echo "Registering license on ${FQDN}... "
    OUTCOME=`curl --silent --show-error --fail -k -X POST ${FQDN}/api/license -H 'Authorization: Bearer '${AUTHTOKEN} -H 'Content-Type: application/json' -H 'Accept: text/plain' -d '{ "license": "'${LICENSE}'", "publicKey": "'${PUBLICKEY}'" }'`

    echo $OUTCOME
fi